#!/usr/bin/env python
# -*- coding: utf-8 -*- 

import rospy

from turtlesim.msg import Pose as TurtlePose
from nav_msgs.msg import Odometry
from geometry_msgs.msg import Pose, Point, Quaternion, TransformStamped

import math
import argparse
import sys
import tf

# Zmienne globalne 
global_turtle_name = "turtle1"  # nazwa żółwia domyślnie ustawiona na: turtle1
global_pose = TurtlePose()      # aktualna pozycja żółwia

global_odom_pub = None          # Publisher topica '/nazwa_żółwia/odom'


# Funkcja inicjalizująca węzeł
def init_node():
    global global_odom_pub

    # Inicjalizacja węzła ROS
    rospy.init_node('turtlesim_bridge', anonymous=True)

    # Publisher publikujący topic '/nazwa_żółwia/odom'
    global_odom_pub = rospy.Publisher('/{}/odom'.format(global_turtle_name), Odometry, queue_size=10)

    # Subskryber topica '/nazwa_żółwia/pose'
    rospy.Subscriber('/{}/pose'.format(global_turtle_name), TurtlePose, pose_callback)


# Funkcja callback wywoływana automatycznie przez Subskrybera
def pose_callback(msg):
    global global_pose
    global_pose = msg

# Funkcja publikująca na podstawie odometrii transformację pomiędzy jego układami współrzędnych: 'base_link' i 'odom'
def publish_tf():
    br = tf.TransformBroadcaster()  # publisher transformacji
    br.sendTransform((global_pose.x,  global_pose.y, 0.0), 
                     tf.transformations.quaternion_from_euler(0.0, 0.0, global_pose.theta),
                     rospy.Time.now(),
                     "base_link",
                     "odom")

    

# Funkcja publikująca na podstawie pozy żółwia jego odometrię
def publish_odom():
    msg = Odometry()
    msg.header.stamp = rospy.Time.now()
    msg.header.frame_id = "odom"
    #msg.pose.pose.position.x = global_pose.x
    #msg.pose.pose.position.y = global_pose.y
    #msg.pose.pose.orientation = tf.transformations.quaternion_from_euler(0.0, 0.0, global_pose.theta)
    msg.pose.pose = Pose(Point(global_pose.x, global_pose.y, 0.0),
                         Quaternion(0.0, 0.0, math.sin(global_pose.theta / 2), math.cos(global_pose.theta / 2)))


    #msg.twist.twist.linear.x = global_pose.linear_velocity
    #msg.twist.twist.angular.z = global_pose.angular_velocity

    global_odom_pub.publish(msg)

def parse_args():
    global global_turtle_name
    # argparse użyte do przyjęcia nazwy żółwia (jest parametrem skryptu)
    parser = argparse.ArgumentParser(description="Interfejs z pakietem turtlesim (publikacja 'odom' na podstawie 'pose' i transformacji 'tf'")
    parser.add_argument('turtle_name', nargs='?', default='turtle1', help="Nazwa żółwia (np. turtle1, turtle2, ...). Domyślnie 'turtle1'.")
    args = parser.parse_args(rospy.myargv()[1:])
    global_turtle_name = args.turtle_name


if __name__ == '__main__':

    try:

        # inicjalizacja
        parse_args()
        init_node()

        # pętla główna
        rate = rospy.Rate(10)  # 10 Hz
        while not rospy.is_shutdown():

            publish_odom()
            publish_tf()

            rate.sleep()  # oczekiwanie na następną iterację
    except rospy.ROSInterruptException:
        pass

