#!/usr/bin/env python
# -*- coding: utf-8 -*- 

import rospy
from turtlesim.msg import Pose
from nav_msgs.msg import Odometry
from geometry_msgs.msg import Pose as GeoPose, Point, Quaternion
import math
import argparse

import tf
from geometry_msgs.msg import TransformStamped

# Zmienna globalna, w której przechowywana będzie aktualna pozycja żółwia
global_pose = Pose()

def publish_tf(odom_x, odom_y, odom_theta):
    br = tf.TransformBroadcaster()
    br.sendTransform(
        (odom_x, odom_y, 0),
        tf.transformations.quaternion_from_euler(0, 0, odom_theta),
        rospy.Time.now(),
        "base_link",
        "odom"
    )

# Funkcja callback, która będzie wywoływana, gdy otrzymamy dane z /turtlesim/pose
def pose_callback(msg):
    global global_pose
    global_pose = msg

def odom_publisher(turtle_name):
    # Inicjalizacja węzła ROS
    rospy.init_node('odom_publisher_from_turtlesim', anonymous=True)

    # Tworzymy publishera, który będzie publikował na temat /turtle1/odom lub /turtle2/odom
    odom_pub = rospy.Publisher('/{}/odom'.format(turtle_name), Odometry, queue_size=10)

    # Subskrybujemy temat /turtle1/pose lub /turtle2/pose w zależności od argumentu
    rospy.Subscriber('/{}/pose'.format(turtle_name), Pose, pose_callback)

    # Pętla główna
    rate = rospy.Rate(10)  # 10 Hz
    while not rospy.is_shutdown():
        # Jeśli mamy dane z /pose, publikujemy odometrię i transformacje
        # Tworzymy wiadomość Odometry
        odom_msg = Odometry()

        # Ustawiamy nagłówek wiadomości
        odom_msg.header.stamp = rospy.Time.now()
        odom_msg.header.frame_id = "odom"

        # Ustawiamy dane o pozycji i orientacji
        odom_msg.pose.pose = GeoPose(Point(global_pose.x, global_pose.y, 0.0),
                                     Quaternion(0.0, 0.0, math.sin(global_pose.theta / 2), math.cos(global_pose.theta / 2)))

        # Ponieważ turtlesim nie dostarcza prędkości w odometrze, możemy ją ustawić na 0
        odom_msg.twist.twist.linear.x = 0.0
        odom_msg.twist.twist.angular.z = 0.0

        # Publikujemy wiadomość na temat /turtle1/odom lub /turtle2/odom w zależności od argumentu
        odom_pub.publish(odom_msg)

        # Publikujemy transformację
        publish_tf(global_pose.x, global_pose.y, global_pose.theta)

        rate.sleep()  # Czekamy na następną iterację

if __name__ == '__main__':
    # Używamy argparse, aby przyjąć nazwę żółwia jako parametr
    parser = argparse.ArgumentParser(description="Publikowanie odometrii na podstawie danych z turtlesim.")
    parser.add_argument('turtle_name', nargs='?', default='turtle1', help="Nazwa żółwia (np. turtle1, turtle2). Domyślnie 'turtle1'.")
    args = parser.parse_args(rospy.myargv()[1:])
    turtle_name = args.turtle_name

    try:
        # Uruchamiamy publishera z nazwą żółwia
        odom_publisher(turtle_name)
    except rospy.ROSInterruptException:
        pass

