#!/usr/bin/env python

import rospy
from geometry_msgs.msg import Twist
import sys, select, termios, tty

msg = """
Sterowanie robotem:
---------------------------
W/S : do przodu/do tyłu
A/D : obrot w lewo/w prawo
Q   : zakoncz
"""

moveBindings = {
    'w': (1, 0),
    's': (-1, 0),
    'a': (0, 1),
    'd': (0, -1),
}

def getKey():
    tty.setraw(sys.stdin.fileno())
    rlist, _, _ = select.select([sys.stdin], [], [], 0.1)
    if rlist:
        key = sys.stdin.read(1)
    else:
        key = ''
    termios.tcsetattr(sys.stdin, termios.TCSADRAIN, settings)
    return key

if __name__ == "__main__":
    settings = termios.tcgetattr(sys.stdin)
    
    rospy.init_node('teleop')
    pub = rospy.Publisher('cmd_vel', Twist, queue_size=1)
    
    speed = 0.5
    turn = 1.0
    
    try:
        print(msg)
        while not rospy.is_shutdown():
            key = getKey()
            if key in moveBindings.keys():
                x = moveBindings[key][0]
                th = moveBindings[key][1]
                
                twist = Twist()
                twist.linear.x = x * speed
                twist.angular.z = th * turn
                pub.publish(twist)
            elif key == 'q':
                break
            else:
                twist = Twist()
                pub.publish(twist)
    except Exception as e:
        print(e)
    
    finally:
        twist = Twist()
        pub.publish(twist)
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, settings)

